# ZPyIrcSimpleBot
#
# Copyright (c) 2003 by Sylvain Hellegouarch.
#
# This software is released under the terms of the GNU General
# Public License version 2, or (at your option) any later
# version. It is distributed WITHOUT ANY WARRANTY. See the GNU
# General Public License for more details.
#

#Zope specific import
from Globals import InitializeClass
from Products.PageTemplates.PageTemplateFile import PageTemplateFile
from OFS.Folder import Folder
from Acquisition import Implicit
from AccessControl import ClassSecurityInfo
from Products.PythonScripts.Utility import allow_module
allow_module('pyirc')
                                           
#Python specific import
import asyncore
import string
import types
from time import strftime,localtime,time

#Let's import the core module
import pyirc

#Manage connexions
_pyirc_object = {}

class ZPyIrcSimpleBotDispatcher(asyncore.dispatcher):
    def __init__(self,server,port,channels,nickname,username,realname,password,host):
        #asyncore.dispatcher.__init__(self)
        self.server = server
        self.nickname = nickname
        self.username = username
        self.realname = realname
        self.password = password
        self.host = host
        self.channels = channels
        self.port = port
        self.connexion = None 
        self.error_messages = {}
        
        #Which scripts to call when we read something ?
        self.bot_scripts = {}

        #Message handler
        self.messages_handler = {}

    def get_errors(self):
        return self.error_messages.items()

    def clear_errors(self):
        self.error_messages = {}

    def register_script(self,id,script):
        self.bot_scripts[id] = script
        self.messages_handler[script.title] = id

    def unregister_script(self,id):
        script = self.bot_scripts[id]
        del self.messages_handler[script.title]
        del self.bot_scripts[id]

    def get_registered_scripts_id(self):
        return self.bot_scripts.keys()

    def is_alive(self):
        return self.connected
    
    def stop(self):
        if self.connexion != None:
            self.connexion.quit()
            self.connexion.disconnect()
            self.del_channel()
            self.connected = 0
            self.connexion = None
            
    def start(self):
        self.connexion = pyirc.IRCServerConnexion(self.server,self.port,self.nickname,self.username,self.realname,self.password,self.host)
        try:
            self.connexion.connect()
            self.connexion.join(self.channels)
        except:
            raise RuntimeError , str("Can't connect to " + self.server + " on port " + str(self.port))

        #Tell asyncore we are connected
        self.set_socket(self.connexion.get_socket())
        self.addr = self.connexion.get_socket().getpeername()
        self.connected = 1

    def join(self,channels):
        self.channels = self.channels + ',' + channels
        if self.connected:
            self.connexion.join(channels)

    def leave(self,channel):
        channels = string.split(self.channels,',')
        channels.remove(channel)
        self.channels = string.join(channels,',')
        if self.connected:
            self.connexion.part(channel)
    
    def _send_response(self,response):
        # Becareful, if your script doesn't send responses in the good shape, it will screw up the bot
        if response[0] == 'privmsg':
            self.connexion.privmsg(response[1],response[2])
        elif response[0] == 'mode':
            self.connexion.mode_channel(response[1],response[2],response[3])
        elif response[0] == 'join':
            self.connexion.join(response[1],response[2])
        elif response[0] == 'part':
            self.connexion.part(response[1],response[2])
        elif response[0] == 'nick':
            self.connexion.nick(response[1])
        elif response[0] == 'quit':
            self.connexion.quit(response[1])
        elif response[0] == 'kick':
            self.connexion.kick(response[1],response[2],response[3])
        elif response[0] == 'topic':
            self.connexion.topic(response[1],response[2])
        elif response[0] == 'ctcp_action':
            self.connexion.ctcp_action(response[1],response[2])
        elif response[0] == 'ctcp_version':
            self.connexion.ctcp_version(response[1],response[2],response[3],response[4])
        elif response[0] == 'ctcp_time':
            self.connexion.ctcp_time(response[1],response[2])
        elif response[0] == 'ctcp_ping':
            self.connexion.ctcp_ping(response[1],response[2])
        elif response[0] == 'away':
            self.connexion.away(response[1])
        elif response[0] == 'invite':
            self.connexion.invite(response[1],response[2])
        elif response[0] == 'info':
            self.connexion.invite(response[1],response[2])
        elif response[0] == 'ison':
            self.connexion.ison(response[1])
        elif response[0] == 'userhost':
            self.connexion.userhost(response[1])
        elif response[1] == 'raw':
            self.connexion.raw(response[1])
        else:
            self.error_messages[int(time())] = str("This message can not be treated : " + str(response))
            
    def handle_connect(self):
        pass

    def handle_error(self):
        self.error_messages[int(time())] = "An unknown error occured with the socket"
        try:
            self.connexion.quit()
            self.connexion.disconnect()
        except:
            pass
        self.del_channel()
        self.connected = 0

    def readable(self):
        return 1

    def handle_read(self):
        self.connexion.process()
        while self.connexion.has_more_messages():
            msg = self.connexion.get_next_message()
            if msg[0] == 'privmsg':
                message = msg[5][:-2]
                command = string.split(message,' ',1)
                if self.messages_handler.has_key(command[0]):
                    id = self.messages_handler[command[0]]
                    m = ""
                    if len(command) == 2:
                        m = command[1]
                    try:
                        # XXX Apply is deprecated since Python 2.3 so is it kept for compatibility ?
                        response = apply(self.bot_scripts[id],(msg[4],m))
                        if response != None:
                            self._send_response(response)
                    except RuntimeError, s:
                        self.error_messages[int(time())] = s
                    except:
                        self.error_messages[int(time())] = "An unknown error occured with the script : " + id
            else:
                if self.messages_handler.has_key(msg[0]):
                    id = self.messages_handler[msg[0]]
                    try:
                        response = apply(self.bot_scripts[id],(msg[1:])) 
                        if response != None:
                            self._send_response(response)
                    except RuntimeError, s:
                        self.error_messages[int(time())] = s
                    except:
                        self.error_messages[int(time())] = "An unknown error occured with the script : " + id
                        
    def writable(self):
        # We never send data this way
        return 0
    
    def handle_write(self):
        pass

    def handle_close(self):
        self.connected = 0
        self.del_channel()
        self.connexion.quit()
        self.connexion.disconnect()
        
manage_addZPyIrcSimpleBotForm = PageTemplateFile('ui/ZPyIrcSimpleBot_add.pt', globals())  
def manage_addZPyIrcSimpleBot(self,id,title,server,port,channels,nickname,username,realname,password='',host='localhost',REQUEST=None):
    """Add ZPyIrcSimpleBot"""
    self._setObject(id, ZPyIrcSimpleBot(id,title,server,port,channels,nickname,username,realname,password,host))
    if REQUEST is not None:
      return self.manage_main(self, REQUEST)
    
class ZPyIrcSimpleBot(Folder):
    meta_type = 'ZPyIrcSimpleBot'

    security = ClassSecurityInfo()
    security.declareObjectProtected('View')
    
    __ac_permissions__=(
        ('Change ZPyIrcSimpleBot', ('index_html')),
    )
    
    security.setPermissionDefault('Change ZPyIrcSimpleBot', ('Manager', 'ZPyIrc Owner'))
    
    index_html = PageTemplateFile('ui/ZPyIrcSimpleBot_index_html.pt', globals())
    
    def __init__(self,id,title,server,port,channels,nickname,username,realname,password,host):
        self.id = id
        self.title = title
        _pyirc_object[self.id] = ZPyIrcSimpleBotDispatcher(server,port,channels,nickname,username,realname,password,host)

    def manage_beforeDelete(self,item,container):
        """Close the bot connexion"""
        _pyirc_object[self.id].stop()
        del  _pyirc_object[self.id]
        
    def get_server(self,REQUEST=None):
        """Return the server name"""
        return _pyirc_object[self.id].server
    
    def get_port(self,REQUEST=None):
        """Return the port"""
        return _pyirc_object[self.id].port

    def get_nickname(self,REQUEST=None):
        """Return the nickname"""
        return _pyirc_object[self.id].nickname
    
    def get_username(self,REQUEST=None):
        """Return the username"""
        return _pyirc_object[self.id].username

    def get_realname(self,REQUEST=None):
        """Return the realname"""
        return _pyirc_object[self.id].realname

    def get_password(self,REQUEST=None):
        """Return the password"""
        return _pyirc_object[self.id].password

    def update_bot(self,REQUEST=None):
        """Update info of the bot"""
        r = self.REQUEST
        _pyirc_object[self.id].server = r['server']
        _pyirc_object[self.id].port = r['port']
        _pyirc_object[self.id].username = r['username']
        _pyirc_object[self.id].nickname = r['nickname']
        _pyirc_object[self.id].realname = r['realname']
        _pyirc_object[self.id].channels = r['channels']
        _pyirc_object[self.id].password = r['password']
        if REQUEST != None:
            return self.REQUEST.RESPONSE.redirect('index_html')

    def get_channels(self):
        """Return the current channels joined by the bot"""
        return _pyirc_object[self.id].channels

    def get_channels_as_list(self):
        """Return the current channels joined by the bot as a list"""
        return string.split(_pyirc_object[self.id].channels,',')
    
    def join_channels(self,REQUEST=None):
        """Make the bot to join some new channels"""
        if self.REQUEST.has_key('channels'):
            _pyirc_object[self.id].join(self.REQUEST['channels'])
        if REQUEST != None:
            return self.REQUEST.RESPONSE.redirect('index_html')

    def leave_channels(self,REQUEST=None):
        """Make the bot to leave some channels"""    
        r = self.REQUEST
        if r.has_key('leave_channel'):
            if type(r['leave_channel']) is types.ListType:
                for channel in r['leave_channel']:
                    _pyirc_object[self.id].leave(channel)
            else:
                _pyirc_object[self.id].leave(r['leave_channel'])
        return self.REQUEST.RESPONSE.redirect('index_html')
    
    def is_alive(self,REQUEST=None):
        """Return the state of the connection"""
        return _pyirc_object[self.id].is_alive()
    
    def stop_connexion(self,REQUEST=None):
        """Stop the bot connexion"""
        _pyirc_object[self.id].stop()
        if REQUEST != None:
            return self.REQUEST.RESPONSE.redirect('index_html')
        
    def start_connexion(self,REQUEST=None):
        """Start the bot connexion"""
        _pyirc_object[self.id].start()
        if REQUEST != None:
            return self.REQUEST.RESPONSE.redirect('index_html')
        
    def get_registered_scripts_id(self,REQUEST=None):
        """Get all registered scripts with the bot"""
        return _pyirc_object[self.id].get_registered_scripts_id()

    def get_non_registered_scripts_id(self,REQUEST=None):
        """Get all the 'not-yet-registered' scripts with the bot"""
        registered = self.get_registered_scripts_id()
        unregistered = []
        for id in self.objectIds(['Script (Python)','External Method']):
            if id not in registered:
                unregistered.append(id)
        return unregistered
    
    def get_error_messages(self,REQUEST=None):
        """Get all errors that occured"""
        errors = []
        for timestamp_id,error in _pyirc_object[self.id].get_errors():
            date = strftime("%a, %d %b %Y %H:%M:%S", localtime(timestamp_id))
            errors.append(str(str(date) + " : " + str(error)))
        return errors.sort()

    def clear_error_messages(self,REQUEST=None):
        """Clear all previously stored error messages"""
        _pyirc_object[self.id].clear_errors()
        if REQUEST != None:
            return self.REQUEST.RESPONSE.redirect('index_html')
        
    def register_scripts(self,REQUEST=None):
        """Register scripts"""
        r = self.REQUEST
        if r.has_key('id_to_add'):
            if type(r['id_to_add']) is types.ListType:
                for id in r['id_to_add']:
                    script = getattr(self,id)
                    _pyirc_object[self.id].register_script(id,script)
            else:
                id = r['id_to_add']
                script = getattr(self,id)
                _pyirc_object[self.id].register_script(id,script)
        return self.REQUEST.RESPONSE.redirect('index_html')
    
    def unregister_scripts(self,REQUEST=None):
        """Unregister scripts from the bot"""
        r = self.REQUEST
        if r.has_key('id_to_add'):
            if type(r['id_to_add']) is types.ListType:
                for id in r['id_to_add']:
                    _pyirc_object[self.id].unregister_script(id)
            else:
                id = r['id_to_add']
                _pyirc_object[self.id].unregister_script(id)
        return self.REQUEST.RESPONSE.redirect('index_html')
    
InitializeClass(ZPyIrcSimpleBot)
